@@ -18,7 +18,6 @@ R_LIB_VERSION_HEADER (r_bin);
 #define R_BIN_SCN_WRITABLE   (1 << 1)
 #define R_BIN_SCN_READABLE   (1 << 2)
 #define R_BIN_SCN_SHAREABLE  (1 << 3)
-#define R_BIN_SCN_MAP        (1 << 4)
 
 #define R_BIN_DBG_STRIPPED 0x01
 #define R_BIN_DBG_STATIC   0x02
@@ -31,6 +30,7 @@ R_LIB_VERSION_HEADER (r_bin);
 #define R_BIN_ENTRY_TYPE_INIT    2
 #define R_BIN_ENTRY_TYPE_FINI    3
 #define R_BIN_ENTRY_TYPE_TLS     4
+#define R_BIN_ENTRY_TYPE_PREINIT 5
 
 #define R_BIN_SIZEOF_STRINGS 512
 #define R_BIN_MAX_ARCH 1024
@@ -66,6 +66,7 @@ R_LIB_VERSION_HEADER (r_bin);
 #define R_BIN_REQ_LISTPLUGINS 0x4000000
 #define R_BIN_REQ_RESOURCES 0x8000000
 #define R_BIN_REQ_INITFINI 0x10000000
+#define R_BIN_REQ_SEGMENTS 0x20000000
 
 /* RBinSymbol->method_flags : */
 #define R_BIN_METH_CLASS 0x0000000000000001L
@@ -91,6 +92,30 @@ R_LIB_VERSION_HEADER (r_bin);
 #define R_BIN_METH_CONSTRUCTOR 0x0000000000100000L
 #define R_BIN_METH_DECLARED_SYNCHRONIZED 0x0000000000200000L
 
+#define R_BIN_BIND_LOCAL_STR "LOCAL"
+#define R_BIN_BIND_GLOBAL_STR "GLOBAL"
+#define R_BIN_BIND_WEAK_STR "WEAK"
+#define R_BIN_BIND_NUM_STR "NUM"
+#define R_BIN_BIND_LOOS_STR "LOOS"
+#define R_BIN_BIND_HIOS_STR "HIOS"
+#define R_BIN_BIND_LOPROC_STR "LOPROC"
+#define R_BIN_BIND_HIPROC_STR "HIPROC"
+#define R_BIN_BIND_UNKNOWN_STR "UNKNOWN"
+
+#define R_BIN_TYPE_NOTYPE_STR "NOTYPE"
+#define R_BIN_TYPE_OBJECT_STR "OBJ"
+#define R_BIN_TYPE_FUNC_STR "FUNC"
+#define R_BIN_TYPE_SECTION_STR "SECT"
+#define R_BIN_TYPE_FILE_STR "FILE"
+#define R_BIN_TYPE_COMMON_STR "COMMON"
+#define R_BIN_TYPE_TLS_STR "TLS"
+#define R_BIN_TYPE_NUM_STR "NUM"
+#define R_BIN_TYPE_LOOS_STR "LOOS"
+#define R_BIN_TYPE_HIOS_STR "HIOS"
+#define R_BIN_TYPE_LOPROC_STR "LOPROC"
+#define R_BIN_TYPE_HIPROC_STR "HIPROC"
+#define R_BIN_TYPE_UNKNOWN_STR "UNK"
+
 enum {
 	R_BIN_SYM_ENTRY,
 	R_BIN_SYM_INIT,
@@ -136,10 +161,16 @@ enum {
 	R_BIN_RELOC_64 = 64
 };
 
+enum {
+	R_BIN_TYPE_DEFAULT = 0,
+	R_BIN_TYPE_CORE = 1
+};
+
 typedef struct r_bin_addr_t {
 	ut64 vaddr;
 	ut64 paddr;
-	ut64 haddr;
+	ut64 hvaddr;
+	ut64 hpaddr;
 	int type;
 	int bits;
 } RBinAddr;
@@ -172,6 +203,7 @@ typedef struct r_bin_info_t {
 	int has_va;
 	int has_pi; // pic/pie
 	int has_canary;
+	int has_retguard;
 	int has_crypto;
 	int has_nx;
 	int big_endian;
@@ -212,16 +244,20 @@ typedef struct r_bin_object_t {
 	RList/*<RBinClass>*/ *classes;
 	RList/*<RBinDwarfRow>*/ *lines;
 	RList/*<??>*/ *mem;	//RBinMem maybe?
+	RList/*<BinMap*/ *maps;
+	char *regstate;
 	RBinInfo *info;
 	RBinAddr *binsym[R_BIN_SYM_LAST];
 	struct r_bin_plugin_t *plugin;
 	int referenced;
 	int lang;
 	Sdb *kv;
+	Sdb *addr2klassmethod;
 	void *bin_obj; // internal pointer used by formats
 } RBinObject;
 
-// XXX: this is a copy of RBinObject
+// XXX: RbinFile may hold more than one RBinObject
+/// XX curplugin == o->plugin
 typedef struct r_bin_file_t {
 	char *file;
 	int fd;
@@ -248,6 +284,15 @@ typedef struct r_bin_file_t {
 	struct r_bin_t *rbin;
 } RBinFile;
 
+typedef struct RBinFileOptions {
+	int rawstr;
+	ut64 baddr; // base address
+	ut64 laddr; // load address
+	ut64 paddr; // offset
+	const char *plugname; // force a plugin? why do i need this?
+	// const char *xtrname;
+} RBinFileOptions;
+
 typedef struct r_bin_t {
 	const char *file;
 	RBinFile *cur;
@@ -260,7 +305,7 @@ typedef struct r_bin_t {
 	ut64 maxstrbuf;
 	int rawstr;
 	Sdb *sdb;
-	RIDPool *file_ids;
+	RIDStorage *ids;
 	RList/*<RBinPlugin>*/ *plugins;
 	RList/*<RBinXtrPlugin>*/ *binxtrs;
 	RList/*<RBinLdrPlugin>*/ *binldrs;
@@ -290,6 +335,7 @@ typedef struct r_bin_xtr_metadata_t {
 	char *libname;
 	char *machine;
 	char *type;
+	const char *xtr_type;
 } RBinXtrMetadata;
 
 typedef int (*FREE_XTR)(void *xtr_obj);
@@ -352,7 +398,8 @@ typedef struct r_bin_plugin_t {
 	int (*fini)(void *user);
 	Sdb * (*get_sdb)(RBinFile *obj);
 	bool (*load)(RBinFile *arch);
-	void *(*load_bytes)(RBinFile *arch, const ut8 *buf, ut64 sz, ut64 loadaddr, Sdb *sdb);
+	void *(*load_bytes)(RBinFile *bf, const ut8 *buf, ut64 sz, ut64 loadaddr, Sdb *sdb);
+	void *(*load_buffer)(RBinFile *bf, RBuffer *buf, ut64 loadaddr, Sdb *sdb);
 	ut64 (*size)(RBinFile *bin); // return ut64 maybe? meh
 	int (*destroy)(RBinFile *arch);
 	bool (*check_bytes)(const ut8 *buf, ut64 length);
@@ -372,6 +419,7 @@ typedef struct r_bin_plugin_t {
 	RList/*<RBinClass>*/* (*classes)(RBinFile *arch);
 	RList/*<RBinMem>*/* (*mem)(RBinFile *arch);
 	RList/*<RBinReloc>*/* (*patch_relocs)(RBin *bin);
+	RList/*<RBinMap>*/* (*maps)(RBinFile *arch);
 	void (*header)(RBinFile *arch);
 	char* (*signature)(RBinFile *arch, bool json);
 	int (*demangle_type)(const char *str);
@@ -382,6 +430,8 @@ typedef struct r_bin_plugin_t {
 	ut64 (*get_vaddr)(RBinFile *arch, ut64 baddr, ut64 paddr, ut64 vaddr);
 	RBuffer* (*create)(RBin *bin, const ut8 *code, int codelen, const ut8 *data, int datalen);
 	char* (*demangle)(const char *str);
+	char* (*regstate)(RBinFile *arch);
+	int (*file_type)(RBinFile *arch);
 	/* default value if not specified by user */
 	int minstrlen;
 	char strfilter;
@@ -402,6 +452,7 @@ typedef struct r_bin_section_t {
 	bool has_strings;
 	bool add; // indicates when you want to add the section to io `S` command
 	bool is_data;
+	bool is_segment;
 } RBinSection;
 
 typedef struct r_bin_class_t {
@@ -515,6 +566,14 @@ typedef struct r_bin_mem_t {	//new toy for esil-init
 	RList *mirrors;		//for mirror access; stuff here should only create new maps not new fds
 } RBinMem;
 
+typedef struct r_bin_map_t {
+	ut64 addr;
+	ut64 offset;
+	int size;
+	int perms;
+	char *file;
+} RBinMap;
+
 typedef struct r_bin_dbginfo_t {
 	int (*get_line)(RBinFile *arch, ut64 addr, char *file, int len, int *line);
 } RBinDbgInfo;
@@ -546,22 +605,47 @@ typedef struct r_bin_bind_t {
 
 #ifdef R_API
 
-/* bin.c */
-R_API void r_bin_load_filter(RBin *bin, ut64 rules);
+/* fd api */
+
+typedef struct r_bin_options_t {
+	ut64 offset; // starting physical address to read from the target file
+	ut64 baseaddr; // where the linker maps the binary in memory
+	ut64 loadaddr; // the desired offset where the binary should be loaded
+	ut64 size; // restrict the size of the target fd
+	int xtr_idx; // load Nth binary
+	int rawstr;
+	int iofd;
+	char *name; // or comment :?
+} RBinOptions;
+
+R_API RBinImport *r_bin_import_clone(RBinImport *o);
+R_API RBinSymbol *r_bin_symbol_clone(RBinSymbol *o);
+
+R_API RBinOptions *r_bin_options_new (ut64 offset, ut64 baddr, int rawstr);
+R_API void r_bin_options_free(RBinOptions *bo);
+R_API int r_bin_open(RBin *bin, const char *filename, RBinOptions *bo);
+R_API RBinFile *r_bin_get_file (RBin *bin, int bd);
+R_API bool r_bin_close(RBin *bin, int bd);
+R_API bool r_bin_query(RBin *bin, const char *query);
+
+/* load */
 R_API int r_bin_load(RBin *bin, const char *file, ut64 baseaddr, ut64 loadaddr, int xtr_idx, int fd, int rawstr);
-R_API int r_bin_reload(RBin *bin, int fd, ut64 baseaddr);
 R_API int r_bin_load_as(RBin *bin, const char *file, ut64 baseaddr, ut64 loadaddr, int xtr_idx, int fd, int rawstr, int fileoffset, const char *name);
 R_API int r_bin_load_io(RBin *bin, int fd, ut64 baseaddr, ut64 loadaddr, int xtr_idx);
-R_API bool r_bin_load_io_at_offset_as(RBin *bin, int fd, ut64 baseaddr, ut64 loadaddr, int xtr_idx, ut64 offset, const char *name);
 R_API int r_bin_load_io_at_offset_as_sz(RBin *bin, int fd, ut64 baseaddr, ut64 loadaddr, int xtr_idx, ut64 offset, const char *name, ut64 sz);
+
+/* bin.c */
+R_API void r_bin_load_filter(RBin *bin, ut64 rules);
+R_API int r_bin_load_languages(RBinFile *binfile);
+
+R_API int r_bin_reload(RBin *bin, int fd, ut64 baseaddr);
 R_API void r_bin_bind(RBin *b, RBinBind *bnd);
 R_API bool r_bin_add(RBin *bin, RBinPlugin *foo);
 R_API bool r_bin_xtr_add(RBin *bin, RBinXtrPlugin *foo);
 R_API bool r_bin_ldr_add(RBin *bin, RBinLdrPlugin *foo);
 R_API void* r_bin_free(RBin *bin);
-R_API int r_bin_load_languages(RBinFile *binfile);
-R_API int r_bin_dump_strings(RBinFile *a, int min);
-R_API RList* r_bin_raw_strings(RBinFile *a, int min);
+R_API RList *r_bin_raw_strings(RBinFile *a, int min);
+R_API int r_bin_dump_strings(RBinFile *a, int min, int raw);
 //io-wrappers
 R_API int r_bin_read_at (RBin *bin, ut64 addr, ut8 *buf, int size);
 R_API int r_bin_write_at (RBin *bin, ut64 addr, const ut8 *buf, int size);
@@ -570,9 +654,11 @@ R_API int r_bin_write_at (RBin *bin, ut64 addr, const ut8 *buf, int size);
 R_API RBinFile *r_bin_file_new(RBin *bin, const char *file, const ut8 *bytes, ut64 sz, ut64 file_sz, int rawstr, int fd, const char *xtrname, Sdb *sdb, bool steal_ptr);
 R_API bool r_bin_file_object_new_from_xtr_data(RBin *bin, RBinFile *bf, ut64 baseaddr, ut64 loadaddr, RBinXtrData *data);
 R_API RBinFile *r_bin_file_new_from_bytes(RBin *bin, const char *file, const ut8 *bytes, ut64 sz, ut64 file_sz, int rawstr, ut64 baseaddr, ut64 loadaddr, int fd, const char *pluginname, const char *xtrname, ut64 offset, bool steal_ptr);
+R_API RBinFile *r_bin_file_new_from_fd(RBin *bin, int fd, RBinFileOptions *options);
 R_API RBinFile *r_bin_file_find_by_arch_bits(RBin *bin, const char *arch, int bits, const char *name);
 R_API RBinObject *r_bin_file_object_find_by_id(RBinFile *binfile, ut32 binobj_id);
-R_API RList *r_bin_file_get_strings(RBinFile *a, int min, int dump);
+R_API RList *r_bin_file_get_strings(RBinFile *a, int min, int dump, int raw);
+R_API void r_bin_file_get_strings_range(RBinFile *bf, RList *list, int min, int raw, ut64 from, ut64 to);
 R_API RBinFile *r_bin_file_find_by_object_id(RBin *bin, ut32 binobj_id);
 R_API RBinFile *r_bin_file_find_by_id(RBin *bin, ut32 binfile_id);
 R_API int r_bin_file_object_add(RBinFile *binfile, RBinObject *o);
@@ -590,7 +676,7 @@ R_API int r_bin_file_ref(RBin *bin, RBinFile *a);
 R_API void r_bin_file_free(void /*RBinFile*/ *bf_);
 R_API RBinFile *r_bin_file_create_append(RBin *bin, const char *file, const ut8 *bytes, ut64 sz, ut64 file_sz, int rawstr, int fd, const char *xtrname, bool steal_ptr);
 R_API RBinFile *r_bin_file_xtr_load_bytes(RBin *bin, RBinXtrPlugin *xtr, const char *filename, const ut8 *bytes, ut64 sz, ut64 file_sz, ut64 baseaddr, ut64 loadaddr, int idx, int fd, int rawstr);
-R_API int r_bin_file_set_bytes(RBinFile *binfile, const ut8 *bytes, ut64 sz, bool steal_ptr);
+R_API bool r_bin_file_set_bytes(RBinFile *binfile, const ut8 *bytes, ut64 sz, bool steal_ptr);
 R_API RBinPlugin *r_bin_file_cur_plugin(RBinFile *binfile);
 
 /* obj.c */
@@ -607,7 +693,6 @@ R_API void r_bin_object_delete_items(RBinObject *o);
 
 // ref
 R_API int r_bin_file_deref_by_bind (RBinBind * binb);
-R_API void r_bin_file_get_strings_range(RBinFile *bf, RList *list, int min, ut64 from, ut64 to);
 R_API int r_bin_file_deref (RBin *bin, RBinFile * a);
 R_API int r_bin_file_ref_by_bind (RBinBind * binb);
 R_API int r_bin_file_ref (RBin *bin, RBinFile * a);
@@ -690,7 +775,7 @@ R_API int r_bin_io_load(RBin *bin, RIO *io, int fd, ut64 baseaddr, ut64 loadaddr
 
 R_API int r_bin_select(RBin *bin, const char *arch, int bits, const char *name);
 R_API int r_bin_select_idx(RBin *bin, const char *name, int idx);
-R_API int r_bin_select_by_ids(RBin *bin, ut32 binfile_id, ut32 binobj_id );
+R_API int r_bin_select_by_ids(RBin *bin, ut32 binfile_id, ut32 binobj_id);
 R_API int r_bin_use_arch(RBin *bin, const char *arch, int bits, const char *name);
 R_API RBinFile * r_bin_file_find_by_arch_bits(RBin *bin, const char *arch, int bits, const char *name);
 R_API void r_bin_list_archs(RBin *bin, int mode);
@@ -759,7 +844,6 @@ extern RBinPlugin r_bin_plugin_mach064;
 extern RBinPlugin r_bin_plugin_mdmp;
 extern RBinPlugin r_bin_plugin_java;
 extern RBinPlugin r_bin_plugin_dex;
-extern RBinPlugin r_bin_plugin_dummy;
 extern RBinPlugin r_bin_plugin_coff;
 extern RBinPlugin r_bin_plugin_ningb;
 extern RBinPlugin r_bin_plugin_ningba;
@@ -769,6 +853,7 @@ extern RBinPlugin r_bin_plugin_xbe;
 extern RBinPlugin r_bin_plugin_bflt;
 extern RBinXtrPlugin r_bin_xtr_plugin_xtr_fatmach0;
 extern RBinXtrPlugin r_bin_xtr_plugin_xtr_dyldcache;
+extern RBinXtrPlugin r_bin_xtr_plugin_xtr_pemixed; 
 extern RBinLdrPlugin r_bin_ldr_plugin_ldr_linux;
 extern RBinPlugin r_bin_plugin_zimg;
 extern RBinPlugin r_bin_plugin_omf;
